#ifndef __CJAMSYSTEM_H__
#define __CJAMSYSTEM_H__


#include "CJamColour.h"
#include "CJamVector2.h"
#include "JamEnums.h"


class CJamSystem
{
public:
	CJamSystem(const char* pszTitle, const int iWidth, const int iHeight, const EJamMouseMode eMouseMode = EJMM_AbsoluteWithSystemCursor);
	~CJamSystem();

	bool RetWantsToExit() const;
	float RetTimeDeltaSinceLastCall();

	CJamVector2 RetMousePositionRelativeSinceLastCall();
	CJamVector2 RetMousePositionAbsolute();
	bool RetIsKeyPressed(const EJamKey eKey);

	class IJamSprite* CreateSprite();
	class IJamSprite* CreateSprite(const CJamVector2& cPos, const EJamAnchor eAnchor, class IJamTexture* pcTexture, const int iFrame = 0);
	void DestroySprite(class IJamSprite* pcSprite);

	class IJamLine* CreateLine();
	class IJamLine* CreateLine(const CJamVector2& cPos1, const CJamVector2& cPos2, const EJamLineCapStyle eCapStyle);
	void DestroyLine(class IJamLine* pcLine);

	class IJamShape* CreateShape();
	class IJamShape* CreateShape(const CJamVector2& cPos, const CJamVector2& cSize, const int iNumSides, const CJamColour& cColour);
	void DestroyShape(class IJamShape* pcShape);

	class IJamRenderTree* CreateRenderTree();
	void DestroyRenderTree(class IJamRenderTree* pcRenderTree);
	void SetRootRenderTree(class IJamRenderTree* pcRenderTree);

	class IJamSystemFont* CreateSystemFont(const int iHeight);
	class IJamSystemText* CreateSystemText();
	class IJamSystemText* CreateSystemText(const CJamVector2& cPos, const EJamAnchor eAnchor, IJamSystemFont* pcFont, const char* pszText);
	void DestroySystemText(class IJamSystemText* pcText);

	class IJamBitmapFont* CreateBitmapFont(class IJamTexture* pcTexture, const int iStartFrame, const CJamVector2& cCharacterSpacing);
	class IJamBitmapText* CreateBitmapText();
	class IJamBitmapText* CreateBitmapText(const CJamVector2& cPos, const EJamAnchor eAnchor, IJamBitmapFont* pcFont, const char* pszText);
	void DestroyBitmapText(class IJamBitmapText* pcText);

	class IJamSound* RetSound(const char* pszFilename);

	class IJamTexture* RetTexture(const char* pszFilename);

	void SetBackgroundColour(const CJamColour& cColour);
	const CJamColour& RetBackgroundColour() const;

	void RenderFrame();

private:
	class CJamRenderManager* mpcRenderManager;
	class CJamSoundManager* mpcSoundManager;
	class CJamTextureManager* mpcTextureManager;

	float iResolution;	// todo: put this into a timer class
	__int64 iStartTime;	// todo: put this into a timer class

	EJamMouseMode meMouseMode;
	bool mbHasFocus;
};


#endif // __CJAMSYSTEM_H__
