#include <stdio.h>
#include "Game.h"
#include "DataIncludes/Textures.h"
#include "GameStates/FrontEnd/CGameState_FrontEnd.h"
#include "GameStates/Game/CGameState_Game.h"


// Pointers to shared stuff
extern CJamSystem* Game_pcSystem = 0;
extern IJamTexture* Game_pcTextures = 0;
extern IJamBitmapFont* Game_pcFont = 0;
extern int Game_iCurrentLevel = 0;
extern int Game_iNumLevels = 0;


// Game states
static CGameState* Game_apcGameStates[Total_EGameState] = 
{
	new CGameState_FrontEnd(),
	new CGameState_Game(),
};
static EGameState Game_eCurrentState = (EGameState)-1;
static EGameState Game_eNextState = (EGameState)-1;
void SetGameState(const EGameState eNewState)
{
	if(Game_eCurrentState != -1)
	{
		Game_apcGameStates[Game_eCurrentState ]->Shutdown();
	}
	Game_eCurrentState  = eNewState;
	Game_apcGameStates[Game_eCurrentState ]->Initialise();
}



//
// Game_Initialise
// Any game initialisation code goes here
//
bool Game_Initialise()
{
	// Load textures
	Game_pcTextures = Game_pcSystem->RetTexture("Textures.jampack");

	// Create a font
	Game_pcFont = Game_pcSystem->CreateBitmapFont(Game_pcSystem->RetTexture("BitmapFontMedium.jampack"), 0, CJamVector2(0, 0));

	// Initialise state machine
	Game_eNextState = EGS_FrontEnd;

	// Start background music
	Game_pcSystem->RetSound("Audio/music.mp3")->PlayLooping();

	// Find out how many levels we have
	Game_iNumLevels = 0;
	do
	{
		char pszFilename[100];
		sprintf_s(pszFilename, 100, "Boards/Board%02d.txt", Game_iNumLevels + 1);
		FILE* fhTest;
		fopen_s(&fhTest, pszFilename, "rt");
		if(fhTest)
		{
			fclose(fhTest);
			Game_iNumLevels++;
		}
		else
		{
			break;
		}
	}while(true);

	return true;
}


//
// Game_Shutdown
// Any game shutdown code goes here
//
bool Game_Shutdown()
{
	return true;
}


//
// Game_Update
// This is called at the frame rate defined by GAME_FPS
//
void Game_Update()
{
	if(Game_eCurrentState != Game_eNextState)
	{
		SetGameState(Game_eNextState);
		Game_eNextState = Game_eCurrentState;
	}
	Game_apcGameStates[Game_eCurrentState ]->Update();
}


//
// Game_RetWantsToExit
// Return true to end the app
//
bool Game_RetWantsToExit()
{
	return false;
}


//
// Set next game state
//
void Game_SetNewState(const EGameState eGameState)
{
	Game_eNextState = eGameState;
}
