#include <math.h>
#include <stdio.h>
#include "DataIncludes/Textures.h"
#include "GameStates/FrontEnd/CGameState_FrontEnd.h"


void CGameState_FrontEnd::Initialise()
{
	// Create a render tree
	mpcRootRenderTree = Game_pcSystem->CreateRenderTree();
	Game_pcSystem->SetRootRenderTree(mpcRootRenderTree);
	Game_pcSystem->SetBackgroundColour(CJamColourARGB(1, 0.75, 0.75, 0.75));

	// Background image
	mpcBackground = Game_pcSystem->CreateSprite(CJamVector2::ZeroVec, EJA_TopLeft, Game_pcTextures, TEXTURES_TITLE_ID);
	mpcBackground->SetDepth(10);
	mpcRootRenderTree->AddRenderObject(mpcBackground);

	// Title text
	mapcTitleText[0] = Game_pcSystem->CreateBitmapText(CJamVector2(320, 50), EJA_Middle, Game_pcFont, "Up The Garden Path");
	mapcTitleText[0]->SetScale(2, 2);
	mapcTitleText[1] = Game_pcSystem->CreateBitmapText(CJamVector2(320, 100), EJA_Middle, Game_pcFont, "A game in a day by Paul Grenfell");
	mapcTitleText[1]->SetScale(0.75, 0.75);
	mapcTitleText[2] = Game_pcSystem->CreateBitmapText(CJamVector2(320, 120), EJA_Middle, Game_pcFont, "www.evilpaul.org");
	mapcTitleText[2]->SetScale(0.75, 0.75);
	mapcTitleText[3] = Game_pcSystem->CreateBitmapText(CJamVector2(320, 600), EJA_Middle, Game_pcFont, "www.dundeejam.com");
	mapcTitleText[3]->SetScale(1.5, 1.5);
	for(int i = 0; i < 4; i++)
	{
		mpcRootRenderTree->AddRenderObject(mapcTitleText[i]);
	}

	// Instrcutions
	for(int i =0; i < 5; i++)
	{
		mapcInstructionText[i] = Game_pcSystem->CreateBitmapText(CJamVector2(320, 200 + (float)i * 30), EJA_Middle, Game_pcFont, "---");
		mpcRootRenderTree->AddRenderObject(mapcInstructionText[i]);
	}
	mapcInstructionText[0]->SetText("Lay a complete path of pieces from");
	mapcInstructionText[1]->SetText("the start arrow to the chequered flag.");
	mapcInstructionText[2]->SetText("Use LEFT mouse button to place piece,");
	mapcInstructionText[3]->SetText("RIGHT mouse button to rotate piece.");
	mapcInstructionText[4]->SetText("Press ESCAPE to quit a puzzle.");

	// Buttons
	mapcButtons[0] = Game_pcSystem->CreateSprite(CJamVector2(200, 500), EJA_Middle, Game_pcTextures, TEXTURES_BUTTON_LEFT_ID);
	mapcButtons[1] = Game_pcSystem->CreateSprite(CJamVector2(320, 500), EJA_Middle, Game_pcTextures, TEXTURES_BUTTON_PLAY_ID);
	mapcButtons[2] = Game_pcSystem->CreateSprite(CJamVector2(440, 500), EJA_Middle, Game_pcTextures, TEXTURES_BUTTON_RIGHT_ID);
	for(int i = 0; i < 3; i++)
	{
		mpcRootRenderTree->AddRenderObject(mapcButtons[i]);
	}

	// Level number
	mpcLevelNumber = Game_pcSystem->CreateBitmapText(CJamVector2(320, 430), EJA_Middle, Game_pcFont, "");
	mpcLevelNumber->SetScale(2, 2);
	mpcRootRenderTree->AddRenderObject(mpcLevelNumber);

	Game_pcSystem->RetSound("Audio/FrontEnd.wav")->Play(0.5f);

	mfTimer = 0.0f;
}


void CGameState_FrontEnd::Shutdown()
{
	Game_pcSystem->DestroyBitmapText(mpcLevelNumber);
	for(int i = 0; i < 4; i++)
	{
		Game_pcSystem->DestroyBitmapText(mapcTitleText[i]);
	}
	for(int i = 0; i < 3; i++)
	{
		Game_pcSystem->DestroySprite(mapcButtons[i]);
	}
	for(int i = 0; i < 5; i++)
	{
		Game_pcSystem->DestroyBitmapText(mapcInstructionText[i]);
	}
	Game_pcSystem->DestroySprite(mpcBackground);
	Game_pcSystem->DestroyRenderTree(mpcRootRenderTree);
}


void CGameState_FrontEnd::Update()
{
	// Update button gfx
	int iSelectedButton = -1;
	for(int i = 0; i < 3; i++)
	{
		if(mapcButtons[i]->RetIsPointInside(Game_pcSystem->RetMousePositionAbsolute()))
		{
			mapcButtons[i]->SetColour(CJamColourARGB::Green);
			iSelectedButton = i;
		}
		else
		{
			mapcButtons[i]->SetColour(CJamColourARGB::Red);
		}
	}

	// Deal with user pressing buttons
	static bool bDebounce = false;
	if(Game_pcSystem->RetIsKeyPressed(EJK_LMOUSEBUTTON))
	{
		if(!bDebounce)
		{
			switch(iSelectedButton)
			{
				// Level down
				case 0:
					Game_iCurrentLevel = Game_iCurrentLevel - 1;
					if(Game_iCurrentLevel < 0)
					{
						Game_iCurrentLevel = Game_iNumLevels - 1;
					}
					break;

				// Play
				case 1:
					Game_SetNewState(EGS_Game);
					break;

				// Level up
				case 2:
					Game_iCurrentLevel = Game_iCurrentLevel + 1;
					if(Game_iCurrentLevel >= Game_iNumLevels)
					{
						Game_iCurrentLevel = 0;
					}
					break;
			}
			Game_pcSystem->RetSound("Audio/Button.wav")->Play(0.5f);
			bDebounce = true;
		}
	}
	else
	{
		bDebounce = false;
	}

	// Show current level number
	char pszBuffer[50];
	sprintf_s(pszBuffer, 50, "Level %d", Game_iCurrentLevel + 1);
	mpcLevelNumber->SetText(pszBuffer);
}

