#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "DataIncludes/Textures.h"
#include "GameStates/Game/CGameState_Game.h"

#define BOARD_WIDTH		(20)
#define BOARD_HEIGHT	(20)

static CGameState_Game::EEndType aeBoard[BOARD_WIDTH * BOARD_HEIGHT];


static const SIntPos aasPieceMasks[4][2] = 
{
	{{0,0},{0,1}},
	{{1,0},{0,0}},
	{{0,1},{0,0}},
	{{0,0},{1,0}},
};

typedef struct
{
	int miTextureID;
	CGameState_Game::EEndType maeEnds[2];
}SPieceDef;
static const SPieceDef aePieceDefs[9] = 
{
	{TEXTURES_TILE_REDRED_ID,		{CGameState_Game::EET_Red,		CGameState_Game::EET_Red}},
	{TEXTURES_TILE_REDGREEN_ID,		{CGameState_Game::EET_Red,		CGameState_Game::EET_Green}},
	{TEXTURES_TILE_REDBLUE_ID,		{CGameState_Game::EET_Red,		CGameState_Game::EET_Blue}},
	{TEXTURES_TILE_GREENRED_ID,		{CGameState_Game::EET_Green,	CGameState_Game::EET_Red}},
	{TEXTURES_TILE_GREENGREEN_ID,	{CGameState_Game::EET_Green,	CGameState_Game::EET_Green}},
	{TEXTURES_TILE_GREENBLUE_ID,	{CGameState_Game::EET_Green,	CGameState_Game::EET_Blue}},
	{TEXTURES_TILE_BLUERED_ID,		{CGameState_Game::EET_Blue,		CGameState_Game::EET_Red}},
	{TEXTURES_TILE_BLUEGREEN_ID,	{CGameState_Game::EET_Blue,		CGameState_Game::EET_Green}},
	{TEXTURES_TILE_BLUEBLUE_ID,		{CGameState_Game::EET_Blue,		CGameState_Game::EET_Blue}},
};


// Read a line of text and strip off any newline characters
static char pszTextBuffer[100 + 1];
void ReadALineOfText(FILE *fhInFile)
{
	fgets(pszTextBuffer, 100, fhInFile);
	for(int i = (int)strlen(pszTextBuffer); i >= 0; i--)
	{
		if(pszTextBuffer[i] < ' ')
		{
			pszTextBuffer[i] = '\0';
		}
		else
		{
			break;
		}
	}
}


void CGameState_Game::Initialise()
{
	// Create a render tree
	mpcRootRenderTree = Game_pcSystem->CreateRenderTree();
	Game_pcSystem->SetRootRenderTree(mpcRootRenderTree);
	Game_pcSystem->SetBackgroundColour(CJamColourARGB(1, 0.75, 0.75, 0.75));

	// Create the cursor
	mpcCursorOutline = Game_pcSystem->CreateSprite();
	mpcCursorOutline->SetDepth(9);
	mpcCursorOutline->SetAnchor(EJA_TopLeft);
	mpcCursorOutline->SetTexture(Game_pcTextures, TEXTURES_CURSOROUTLINE_ID);
	mpcCursorOutline->SetIsVisible(false);
	mpcRootRenderTree->AddRenderObject(mpcCursorOutline);

	// Some text items
	for(int i = 0; i < 3; i++)
	{
		mapcIntroText[i] = Game_pcSystem->CreateBitmapText(CJamVector2(320.0f, 320.0f + 75.0f * (i -1)), EJA_Middle, Game_pcFont, "---");
		mapcIntroText[i]->SetDepth(5);
		mapcIntroText[i]->SetScale(CJamVector2(1, 1));
		mpcRootRenderTree->AddRenderObject(mapcIntroText[i]);
	}

	// UI text
	mpcUiTextRenderTree = Game_pcSystem->CreateRenderTree();
	mpcUiTextRenderTree->SetDepth(15);
	mpcNextPieceText = Game_pcSystem->CreateBitmapText(CJamVector2(-100, 0), EJA_Middle, Game_pcFont, "Next");
	mpcUiTextRenderTree->AddRenderObject(mpcNextPieceText);
	mpcMovesTextTitle = Game_pcSystem->CreateBitmapText(CJamVector2(-100, 0), EJA_Middle, Game_pcFont, "Moves");
	mpcUiTextRenderTree->AddRenderObject(mpcMovesTextTitle);
	mpcMovesTextCount = Game_pcSystem->CreateBitmapText(CJamVector2(-100, 0), EJA_Middle, Game_pcFont, "0");
	mpcUiTextRenderTree->AddRenderObject(mpcMovesTextCount);
	mpcParTextTitle = Game_pcSystem->CreateBitmapText(CJamVector2(-100, 0), EJA_Middle, Game_pcFont, "Par");
	mpcUiTextRenderTree->AddRenderObject(mpcParTextTitle);
	mpcParTextCount = Game_pcSystem->CreateBitmapText(CJamVector2(-100, 0), EJA_Middle, Game_pcFont, "0");
	mpcUiTextRenderTree->AddRenderObject(mpcParTextCount);
	mpcRootRenderTree->AddRenderObject(mpcUiTextRenderTree);
	mpcUiTextRenderTree->SetIsVisible(false);

	// Next piece indicator
	mpcNextPiece = Game_pcSystem->CreateSprite();
	mpcNextPiece->SetAnchor(EJA_TopMiddle);
	mpcNextPiece->SetPosition(-100, 0);
	mpcUiTextRenderTree->AddRenderObject(mpcNextPiece);


	// Load the level file
	char pszLevelFilename[100];
	sprintf_s(pszLevelFilename, 100, "Boards/Board%02d.txt", Game_iCurrentLevel + 1);
	FILE* fhLevel;
	fopen_s(&fhLevel, pszLevelFilename, "rt");
	JamAssert(fhLevel, "Couldn't find level file");

	// Background texture
	ReadALineOfText(fhLevel);
	mpcBackgroundImage = Game_pcSystem->CreateSprite(CJamVector2::ZeroVec, EJA_TopLeft, Game_pcSystem->RetTexture(pszTextBuffer));
	mpcBackgroundImage->SetDepth(20);
	mpcRootRenderTree->AddRenderObject(mpcBackgroundImage);

	// Random seed
	ReadALineOfText(fhLevel);
	int iSeed = 0;
	for(int i = 0; i < (int)strlen(pszTextBuffer); i++)
	{
		iSeed += pszTextBuffer[i];
	}
	srand(iSeed);
	miNextPieceType = rand() % 9;

	// Intro text
	for(int i = 0; i < 3; i++)
	{
		ReadALineOfText(fhLevel);
		mapcIntroText[i]->SetText(pszTextBuffer);
	}

	// Par moves
	ReadALineOfText(fhLevel);
	const int iParMoves = atoi(pszTextBuffer);
	sprintf_s(pszTextBuffer, "%d", iParMoves);
	mpcParTextCount->SetText(pszTextBuffer);

	// Load the level map
	for(int i = 0; i < BOARD_HEIGHT; i++)
	{
		fgets(pszTextBuffer, 100, fhLevel);
		for(int j = 0; j < BOARD_WIDTH; j++)
		{
			const char iType = pszTextBuffer[j];
			EEndType eEndType = EET_Barrier;
			switch(iType)
			{
				case '-':
					eEndType = EET_None;
					break;
				case '#':
					break;
				case 'N':
					mpcNextPieceText->SetPosition((float)(j * 32 + 16), (float)(i * 32 - 16));
					mpcNextPiece->SetPosition((float)(j * 32 + 16), (float)(i * 32));
					break;
				case 'M':
					mpcMovesTextTitle->SetPosition((float)(j * 32 + 16), (float)(i * 32 - 16));
					mpcMovesTextCount->SetPosition((float)(j * 32 + 16), (float)(i * 32 + 16));
					break;
				case 'P':
					mpcParTextTitle->SetPosition((float)(j * 32 + 16), (float)(i * 32 - 16));
					mpcParTextCount->SetPosition((float)(j * 32 + 16), (float)(i * 32 + 16));
					break;
				case 's':
					eEndType = EET_Start;
					break;
				case 'e':
					eEndType = EET_End;
					break;
			}
			aeBoard[j + i * BOARD_WIDTH] = eEndType;
		}
	}

	fclose(fhLevel);

	Game_pcSystem->RetSound("Audio/GameIntro.wav")->Play(0.5f);
	meCurrentState = ES_Intro;
	miStateTimer = 0;
}


void CGameState_Game::Shutdown()
{
	for(TList<IJamSprite*>::CIterator cIterator = mcPieces.Begin();
		cIterator != mcPieces.End();
		cIterator.Next())
	{
		Game_pcSystem->DestroySprite(cIterator.Ret());
	}
	while(!mcPieces.RetIsEmpty())
	{
		mcPieces.PopBack();
	}
	Game_pcSystem->DestroyBitmapText(mpcParTextTitle);
	Game_pcSystem->DestroyBitmapText(mpcParTextCount);
	Game_pcSystem->DestroyBitmapText(mpcMovesTextTitle);
	Game_pcSystem->DestroyBitmapText(mpcMovesTextCount);
	Game_pcSystem->DestroyBitmapText(mpcNextPieceText);
	for(int i = 0; i < 3; i++)
	{
		Game_pcSystem->DestroyBitmapText(mapcIntroText[i]);
	}
	Game_pcSystem->DestroySprite(mpcBackgroundImage);
	Game_pcSystem->DestroySprite(mpcCursorOutline);
	Game_pcSystem->DestroySprite(mpcCurrentPiece);
	Game_pcSystem->DestroySprite(mpcNextPiece);
	Game_pcSystem->DestroyRenderTree(mpcUiTextRenderTree);
	Game_pcSystem->DestroyRenderTree(mpcRootRenderTree);
}


void CGameState_Game::Update()
{
	switch(meCurrentState)
	{
		case ES_Intro:
			{
				const float fFontScale = sinf((float)miStateTimer * 0.05f) * 0.25f + 1.5f;
				for(int i = 0; i < 3; i++)
				{
					mapcIntroText[i]->SetScale(fFontScale, fFontScale);
				}
				miStateTimer++;
				if(miStateTimer == 200)
				{
					for(int i = 0; i < 3; i++)
					{
						mapcIntroText[i]->SetIsVisible(false);
					}
					mpcCursorOutline->SetIsVisible(true);
					mpcUiTextRenderTree->SetIsVisible(true);
					NewPiece();
					Game_pcSystem->RetSound("Audio/GameStart.wav")->Play(0.5f);

					meCurrentState = ES_Game;
					miStateTimer = 0;
				}
			}
			break;

		case ES_Game:
			{
				// Position piece and cursor on the screen
				const SIntPos sCurrentPiecePos = GetPiecePosFromMouse();
				const CJamVector2 sCurrentPieceScreenPos = PiecePosToScreenPos(sCurrentPiecePos, miCurrentPieceRotation);
				mpcCurrentPiece->SetPosition(sCurrentPieceScreenPos);
				switch(miCurrentPieceRotation)
				{
					case 0:
						mpcCursorOutline->SetPosition(sCurrentPieceScreenPos + CJamVector2(-4, -4));
						break;
					case 1:
						mpcCursorOutline->SetPosition(sCurrentPieceScreenPos + CJamVector2(+4, -4));
						break;
					case 2:
						mpcCursorOutline->SetPosition(sCurrentPieceScreenPos + CJamVector2(+4, +4));
						break;
					case 3:
						mpcCursorOutline->SetPosition(sCurrentPieceScreenPos + CJamVector2(-4, +4));
						break;
				}

				// Colour the cursor depending on whether this is a valid place to drop the current piece
				if(CanPlacePieceHere(sCurrentPiecePos, miCurrentPieceRotation))
				{
					mpcCursorOutline->SetColour(CJamColourARGB::Green);
				}
				else
				{
					mpcCursorOutline->SetColour(CJamColourARGB::Red);
				}

				// Check for input
				if(Game_pcSystem->RetIsKeyPressed(EJK_RMOUSEBUTTON))
				{
					if(!mbKeyDebounce)
					{
						// Rotate piece
						miCurrentPieceRotation = (miCurrentPieceRotation + 1) % 4;
						mpcCurrentPiece->SetRotation(miCurrentPieceRotation ? 3.1415962f / 2 * (float)miCurrentPieceRotation: 0);
						mpcCursorOutline->SetRotation(miCurrentPieceRotation ? 3.1415962f / 2 * (float)miCurrentPieceRotation: 0);
						mbKeyDebounce = true;
						Game_pcSystem->RetSound("Audio/Rotate.wav")->Play(0.5f);
					}
				}
				else if(Game_pcSystem->RetIsKeyPressed(EJK_LMOUSEBUTTON))
				{
					if(!mbKeyDebounce)
					{
						// Place piece
						if(CanPlacePieceHere(sCurrentPiecePos, miCurrentPieceRotation))
						{
							AddPieceToBoard(sCurrentPiecePos, miCurrentPieceType, miCurrentPieceRotation);
							NewPiece();
							Game_pcSystem->RetSound("Audio/Place.wav")->Play(0.5f);
							sprintf_s(pszTextBuffer, "%d", mcPieces.RetSize());
							mpcMovesTextCount->SetText(pszTextBuffer);
						}
						else
						{
							Game_pcSystem->RetSound("Audio/NoPlace.wav")->Play(0.5f);
						}
						mbKeyDebounce = true;
					}
				}
				else
				{
					mbKeyDebounce = false;
				}
			}
			break;

		case ES_Complete:
			{
				for(int i = 0; i < 3; i++)
				{
					const float fFontScale = sinf((float)(miStateTimer + i * 40) * 0.1f) + 1.5f;
					const float fFontRotation = sinf((float)(miStateTimer + i * 50) * 0.2f) * 0.15f;
					mapcIntroText[i]->SetScale(fFontScale, fFontScale);
					mapcIntroText[i]->SetRotation(fFontRotation);
				}
				miStateTimer++;
				if(miStateTimer == 200)
				{
					// Advance a level
					Game_iCurrentLevel = Game_iCurrentLevel + 1;
					if(Game_iCurrentLevel >= Game_iNumLevels)
					{
						Game_iCurrentLevel = 0;
					}

					Game_SetNewState(EGS_FrontEnd);
				}
			}
			break;
	}

	// Let user get back to the front end
	if(Game_pcSystem->RetIsKeyPressed(EJK_ESCAPE))
	{
		Game_SetNewState(EGS_FrontEnd);
	}
}


SIntPos CGameState_Game::GetPiecePosFromMouse()
{
	SIntPos sPos;
	CJamVector2 cPos = Game_pcSystem->RetMousePositionAbsolute();
	sPos.miX = (int)(cPos.mfX + 0) / 32;
	sPos.miY = (int)(cPos.mfY + 0) / 32;
	return sPos;
}


void CGameState_Game::NewPiece()
{
	miCurrentPieceType = miNextPieceType;
	miNextPieceType = rand() % 9;
	mpcNextPiece->SetTexture(Game_pcTextures, aePieceDefs[miNextPieceType].miTextureID);

	mpcCurrentPiece = Game_pcSystem->CreateSprite();
	mpcCurrentPiece->SetDepth(9);
	mpcCurrentPiece->SetTexture(Game_pcTextures, aePieceDefs[miCurrentPieceType].miTextureID);
	mpcCurrentPiece->SetAnchor(EJA_TopLeft);
	mpcCurrentPiece->SetPosition(-100, 0);
	mpcRootRenderTree->AddRenderObject(mpcCurrentPiece);
	miCurrentPieceRotation = 0;
	mpcCursorOutline->SetRotation(0);
}


bool CGameState_Game::CanPlacePieceHere(const SIntPos& sPos, const int iRotation)
{
	// Check for board edges
	switch(iRotation)
	{
		case 0:
		case 2:
			if(sPos.miX < 0 || sPos.miX > BOARD_WIDTH - 1)
			{
				return false;
			}
			if(sPos.miY < 0 || sPos.miY > BOARD_HEIGHT - 2)
			{
				return false;
			}
			break;

		case 1:
		case 3:
			if(sPos.miX < 0 || sPos.miX > BOARD_WIDTH - 2)
			{
				return false;
			}
			if(sPos.miY < 0 || sPos.miY > BOARD_HEIGHT - 1)
			{
				return false;
			}
			break;
	}

	bool bOnStartPiece = false;
	const SIntPos* piCurrentPieceMasks = aasPieceMasks[iRotation];
	for(int i = 0; i < 2; i++)
	{
		SIntPos sTestPos;
		sTestPos.miX = piCurrentPieceMasks[i].miX + sPos.miX;
		sTestPos.miY = piCurrentPieceMasks[i].miY + sPos.miY;

		// Over the top of an existing piece?
		const EEndType eBoardEndType = aeBoard[sTestPos.miX + sTestPos.miY * BOARD_WIDTH];
		if(eBoardEndType != EET_None && eBoardEndType != EET_Start && eBoardEndType != EET_End)
		{
			return false;
		}
		if(eBoardEndType == EET_Start)
		{
			bOnStartPiece = true;
		}

		// Can the colour be placed here?
		if(!CheckEndTypeAgainstBoard(sTestPos, aePieceDefs[miCurrentPieceType].maeEnds[i], false))
		{
			return false;
		}
	}

	if(!bOnStartPiece)
	{
		int iCount = 0;
		for(int i = 0; i < 2; i++)
		{
			SIntPos sTestPos;
			sTestPos.miX = piCurrentPieceMasks[i].miX + sPos.miX;
			sTestPos.miY = piCurrentPieceMasks[i].miY + sPos.miY;
		
			// Can the colour be placed here?
			if(CheckEndTypeAgainstBoard(sTestPos, aePieceDefs[miCurrentPieceType].maeEnds[i], !bOnStartPiece))
			{
				iCount++;
			}
		}
		if(iCount == 0)
		{
			return false;
		}
	}
	return true;
}


bool CGameState_Game::CheckEndTypeAgainstBoard(const SIntPos& sPos, const EEndType eEndType, const bool bMustMatch)
{
	static const SIntPos asTestOffsets[4] = 
	{
		{-1, 0},
		{+1, 0},
		{0, -1},
		{0, +1},
	};
	bool bNextToSameColour = false;
	for(int i = 0; i < 4; i++)
	{
		SIntPos sTestPos;
		sTestPos.miX = sPos.miX + asTestOffsets[i].miX;
		sTestPos.miY = sPos.miY + asTestOffsets[i].miY;
		if(sTestPos.miX >= 0 && sTestPos.miX < BOARD_WIDTH &&
		   sTestPos.miY >= 0 && sTestPos.miY < BOARD_HEIGHT)
		{
			const EEndType eBoardEndType = aeBoard[sTestPos.miX + sTestPos.miY * BOARD_WIDTH];
			if(eBoardEndType != EET_None && eBoardEndType != EET_Start && eBoardEndType != EET_End && eBoardEndType != EET_Barrier && eBoardEndType != eEndType)
			{
				return false;
			}
			if(eBoardEndType == eEndType)
			{
				bNextToSameColour = true;
			}
		}
	}

	if(bMustMatch)
	{
		return bNextToSameColour;
	}
	else
	{
		return true;
	}
}


void CGameState_Game::AddPieceToBoard(const SIntPos& sPos, const int iPieceType, const int iRotation)
{
	// Mark it on the board
	const SIntPos* piCurrentPieceMasks = aasPieceMasks[iRotation];
	const SPieceDef* psPieceDef = &aePieceDefs[iPieceType];
	for(int i = 0; i < 2; i++)
	{
		SIntPos sTestPos;
		sTestPos.miX = piCurrentPieceMasks[i].miX + sPos.miX;
		sTestPos.miY = piCurrentPieceMasks[i].miY + sPos.miY;
		if(aeBoard[sTestPos.miX + sTestPos.miY * BOARD_WIDTH] == EET_End)
		{
			// Reached an end position!
			mpcUiTextRenderTree->SetIsVisible(false);
			mpcCurrentPiece->SetIsVisible(false);
			mpcCursorOutline->SetIsVisible(false);

			mapcIntroText[0]->SetText("You did it!");
			mapcIntroText[1]->SetText("You did it!");
			mapcIntroText[2]->SetText("You did it!");
			for(int i = 0; i < 3; i++)
			{
				mapcIntroText[i]->SetIsVisible(true);
			}
			Game_pcSystem->RetSound("Audio/Complete.wav")->Play(0.5f);

			meCurrentState = ES_Complete;
			miStateTimer = 0;
		}
		aeBoard[sTestPos.miX + sTestPos.miY * BOARD_WIDTH] = psPieceDef->maeEnds[i];
	}

	// Add the current piece to the list of placed pieces
	mpcCurrentPiece->SetColour(CJamColourARGB::White);
	mpcCurrentPiece->SetDepth(10);
	mcPieces.PushBack(mpcCurrentPiece);
	mpcCurrentPiece = 0;
}


CJamVector2 CGameState_Game::PiecePosToScreenPos(const SIntPos& sPos, const int iRotation)
{
	CJamVector2 cPos;
	cPos = CJamVector2((float)sPos.miX * 32, (float)sPos.miY * 32);
	switch(iRotation)
	{
		case 0:
			break;
		case 1:
			cPos.mfX += 32 * 2;
			break;
		case 2:
			cPos.mfX += 32 * 1;
			cPos.mfY += 32 * 2;
			break;
		case 3:
			cPos.mfY += 32 * 1;
			break;
	}
	return cPos;
}
