#ifndef __CGAMESTATE_GAME_H__
#define __CGAMESTATE_GAME_H__


#include "GameStates/CGameState.h"
#include "Utils/TList.h"

typedef struct
{
	int miX, miY;
}SIntPos;


class CGameState_Game : public CGameState
{
public:
	virtual void Initialise();
	virtual void Shutdown();
	virtual void Update();

	typedef enum
	{
		EET_None,
		EET_Start,
		EET_End,
		EET_Barrier,
		EET_Red,
		EET_Green,
		EET_Blue,
	}EEndType;
private:
	typedef enum
	{
		ES_Intro,
		ES_Game,
		ES_Complete,
	}EState;
	EState meCurrentState;
	int miStateTimer;


	void ClearBoard();
	SIntPos GetPiecePosFromMouse();
	void NewPiece();
	bool CanPlacePieceHere(const SIntPos& sPos, const int iRotation);
	bool CheckEndTypeAgainstBoard(const SIntPos& sPos, const EEndType eEndType, const bool bMustMatch);
	void AddPieceToBoard(const SIntPos& sPos, const int iPieceType, const int iRotation);
	CJamVector2 PiecePosToScreenPos(const SIntPos& sPos, const int iRotation);

	IJamRenderTree* mpcRootRenderTree;
	IJamRenderTree* mpcUiTextRenderTree;

	IJamSprite* mpcBackgroundImage;

	IJamSprite* mpcCursorOutline;
	IJamSprite* mpcCurrentPiece;
	int miCurrentPieceRotation;
	int miCurrentPieceType;
	int miNextPieceType;
	IJamSprite* mpcNextPiece;

	TList<IJamSprite*> mcPieces;

	IJamBitmapText* mapcIntroText[3];
	IJamBitmapText* mpcNextPieceText;
	IJamBitmapText* mpcMovesTextTitle;
	IJamBitmapText* mpcMovesTextCount;
	IJamBitmapText* mpcParTextTitle;
	IJamBitmapText* mpcParTextCount;

	bool mbKeyDebounce;
};


#endif // __CGAMESTATE_GAME_H__
