#ifndef __TLIST_H__
#define __TLIST_H__


#include <assert.h>


template <typename Type>
class TList
{
private:
	typedef struct _SListNode
	{
		Type mData;
		_SListNode* mpsPrev;
		_SListNode* mpsNext;
	}SListNode;


public:
	class CIterator
	{
	public:
		CIterator() :
			mpsNode(0)
		{
		}
		CIterator(SListNode* psStartNode) :
			mpsNode(psStartNode)
		{
			assert(mpsNode);
		}
		~CIterator()
		{
			assert(mpsNode);
		}

		bool operator == (const CIterator& Test) const {return mpsNode == Test.mpsNode;}
		bool operator != (const CIterator& Test) const {return mpsNode != Test.mpsNode;}

		Type Ret()
		{
			assert(mpsNode);
			return mpsNode->mData;
		}
		Type Next()
		{
			assert(mpsNode);
			mpsNode = mpsNode->mpsNext;
			return mpsNode->mData;
		}
		Type Prev()
		{
			assert(mpsNode);
			mpsNode = mpsNode->mpsPrev;
			return mpsNode->mData;
		}

	private:
		SListNode* mpsNode;
	};


	class CConstIterator
	{
	public:
		CConstIterator() :
			mpsNode(0)
		{
		}
		CConstIterator(SListNode* psStartNode) :
			mpsNode(psStartNode)
		{
			assert(mpsNode);
		}
		~CConstIterator()
		{
			assert(mpsNode);
		}

		bool operator == (const CConstIterator& Test) const {return mpsNode == Test.mpsNode;}
		bool operator != (const CConstIterator& Test) const {return mpsNode != Test.mpsNode;}

		const Type Ret()
		{
			assert(mpsNode);
			return mpsNode->mData;
		}
		const Type Next()
		{
			assert(mpsNode);
			mpsNode = mpsNode->mpsNext;
			return mpsNode->mData;
		}
		const Type Prev()
		{
			assert(mpsNode);
			mpsNode = mpsNode->mpsPrev;
			return mpsNode->mData;
		}

	private:
		SListNode* mpsNode;
	};


public:
	TList() :
	  mpsNode(0),
	  miSize(0)
	{
		mpsNode = new SListNode;
		mpsNode->mpsPrev = mpsNode;
		mpsNode->mpsNext = mpsNode;
	}
	~TList()
	{
		Clear();
		delete mpsNode;
	}

	Type Front() {return *Begin();}
	const Type Front() const {return *Begin();}
	Type Back() {return *End();}
	const Type Back() const {return *End();}
	CIterator Begin() {return CIterator(mpsNode->mpsNext);}
	CConstIterator Begin() const {return CConstIterator(mpsNode->mpsNext);}
	CIterator End() {return CIterator(mpsNode);}
	CConstIterator End() const {return CConstIterator(mpsNode);}

	void PushFront(const Type& Data)
	{
		InsertBefore(mpsNode->mpsNext, Data);
	}
	void PushBack(const Type& Data)
	{
		InsertBefore(mpsNode, Data);
	}
	void PopFront()
	{
		Erase(mpsNode->mpsNext);
	}
	void PopBack()
	{
		Erase(mpsNode->mpsPrev);
	}
	void Remove(const Type& Data)		// not heavily tested
	{
		SListNode* psNode = mpsNode;
		while(mpsNode->mpsNext != mpsNode)
		{
			if(psNode->mData == Data)
			{
				Erase(psNode);
				return;
			}
			psNode = psNode->mpsNext;
		}
	}
	void Clear()
	{
		while(mpsNode->mpsNext != mpsNode)
		{
			Erase(mpsNode->mpsNext);
		}
		assert(mpsNode->mpsPrev == mpsNode);
		assert(mpsNode->mpsNext == mpsNode);
		assert(miSize == 0);
	}

	int RetSize() const
	{
		return miSize;
	}
	bool RetIsEmpty() const
	{
		return miSize == 0;
	}

private:
	void InsertBefore(SListNode* psNode, const Type& Data)
	{
		SListNode* psNewNode = new SListNode;
		psNewNode->mData = Data;
		psNewNode->mpsNext = psNode;
		psNewNode->mpsPrev = psNode->mpsPrev;
		psNode->mpsPrev->mpsNext = psNewNode;
		psNode->mpsPrev = psNewNode;
		miSize++;
	}
	void Erase(SListNode* psNode)
	{
		assert(psNode != mpsNode);
		assert(miSize > 0);
		SListNode* psPrev = psNode->mpsPrev;
		SListNode* psNext = psNode->mpsNext;
		psPrev->mpsNext = psNext;
		psNext->mpsPrev = psPrev;
		delete psNode;
		miSize--;
	}

	SListNode* mpsNode;
	int miSize;

};


#endif // __TLIST_H__
